@@ -298,8 +298,7 @@ impl PhysicalExpr for BinaryExpr {
                 AggState::NotAggregated(_) => {
                     has_not_agg = true;
                     if let Some(p) = previous {
-                        not_agg_groups_may_diverge |=
-                            !std::ptr::eq(p.groups.as_ref(), ac.groups.as_ref());
+                        not_agg_groups_may_diverge |= !p.groups.is_same(&ac.groups)
                     }
                     previous = Some(ac);
                     if ac.groups.is_overlapping() {
