@@ -58,27 +58,22 @@ impl std::fmt::Write for IgnoreFmt {
 
 pub(super) trait Serializer<'a> {
     fn serialize(&mut self, buf: &mut Vec<u8>, options: &SerializeOptions);
-    // Updates the array without changing the configuration.
-    fn update_array(&mut self, array: &'a dyn Array);
 }
 
 fn make_serializer<'a, T, I: Iterator<Item = Option<T>>, const QUOTE_NON_NULL: bool>(
     f: impl FnMut(T, &mut Vec<u8>, &SerializeOptions),
     iter: I,
-    update_array: impl FnMut(&'a dyn Array) -> I,
 ) -> impl Serializer<'a> {
-    struct SerializerImpl<F, I, Update, const QUOTE_NON_NULL: bool> {
+    struct SerializerImpl<F, I, const QUOTE_NON_NULL: bool> {
         f: F,
         iter: I,
-        update_array: Update,
     }
 
-    impl<'a, T, F, I, Update, const QUOTE_NON_NULL: bool> Serializer<'a>
-        for SerializerImpl<F, I, Update, QUOTE_NON_NULL>
+    impl<'a, T, F, I, const QUOTE_NON_NULL: bool> Serializer<'a>
+        for SerializerImpl<F, I, QUOTE_NON_NULL>
     where
         F: FnMut(T, &mut Vec<u8>, &SerializeOptions),
         I: Iterator<Item = Option<T>>,
-        Update: FnMut(&'a dyn Array) -> I,
     {
         fn serialize(&mut self, buf: &mut Vec<u8>, options: &SerializeOptions) {
             let item = self.iter.next().expect(TOO_MANY_MSG);
@@ -95,17 +90,9 @@ fn make_serializer<'a, T, I: Iterator<Item = Option<T>>, const QUOTE_NON_NULL: b
                 None => buf.extend_from_slice(options.null.as_bytes()),
             }
         }
-
-        fn update_array(&mut self, array: &'a dyn Array) {
-            self.iter = (self.update_array)(array);
-        }
     }
 
-    SerializerImpl::<_, _, _, QUOTE_NON_NULL> {
-        f,
-        iter,
-        update_array,
-    }
+    SerializerImpl::<_, _, QUOTE_NON_NULL> { f, iter }
 }
 
 fn integer_serializer<I: NativeType + itoa::Integer>(
@@ -117,18 +104,12 @@ fn integer_serializer<I: NativeType + itoa::Integer>(
         buf.extend_from_slice(value.as_bytes());
     };
 
-    make_serializer::<_, _, false>(f, array.iter(), |array| {
-        array
-            .as_any()
-            .downcast_ref::<PrimitiveArray<I>>()
-            .expect(ARRAY_MISMATCH_MSG)
-            .iter()
-    })
+    make_serializer::<_, _, false>(f, array.iter())
 }
 
 fn float_serializer_no_precision_autoformat_f16(array: &Float16Array) -> impl Serializer<'_> {
     let f = move |&item, buf: &mut Vec<u8>, _options: &SerializeOptions| {
-        let mut buffer = ryu::Buffer::new();
+        let mut buffer = zmij::Buffer::new();
         let cast: f32 = NumCast::from(item).unwrap();
         let value = buffer.format(cast);
         buf.extend_from_slice(value.as_bytes());
@@ -136,11 +117,11 @@ fn float_serializer_no_precision_autoformat_f16(array: &Float16Array) -> impl Se
     float_serializer_no_precision_autoformat_(array, f)
 }
 
-fn float_serializer_no_precision_autoformat<I: NativeType + ryu::Float>(
+fn float_serializer_no_precision_autoformat<I: NativeType + zmij::Float>(
     array: &PrimitiveArray<I>,
 ) -> impl Serializer<'_> {
     let f = move |&item, buf: &mut Vec<u8>, _options: &SerializeOptions| {
-        let mut buffer = ryu::Buffer::new();
+        let mut buffer = zmij::Buffer::new();
         let value = buffer.format(item);
         buf.extend_from_slice(value.as_bytes());
     };
@@ -155,20 +136,14 @@ fn float_serializer_no_precision_autoformat_<
     array: &'a PrimitiveArray<I>,
     f: F,
 ) -> impl Serializer<'a> {
-    make_serializer::<_, _, false>(f, array.iter(), |array| {
-        array
-            .as_any()
-            .downcast_ref::<PrimitiveArray<I>>()
-            .expect(ARRAY_MISMATCH_MSG)
-            .iter()
-    })
+    make_serializer::<_, _, false>(f, array.iter())
 }
 
 fn float_serializer_no_precision_autoformat_decimal_comma_f16(
     array: &Float16Array,
 ) -> impl Serializer<'_> {
     let f = move |&item, buf: &mut Vec<u8>, _options: &SerializeOptions| {
-        let mut buffer = ryu::Buffer::new();
+        let mut buffer = zmij::Buffer::new();
         let cast: f32 = NumCast::from(item).unwrap();
         let value = buffer.format(cast);
 
@@ -179,11 +154,11 @@ fn float_serializer_no_precision_autoformat_decimal_comma_f16(
     float_serializer_no_precision_autoformat_decimal_comma_(array, f)
 }
 
-fn float_serializer_no_precision_autoformat_decimal_comma<I: NativeType + ryu::Float>(
+fn float_serializer_no_precision_autoformat_decimal_comma<I: NativeType + zmij::Float>(
     array: &PrimitiveArray<I>,
 ) -> impl Serializer<'_> {
     let f = move |&item, buf: &mut Vec<u8>, _options: &SerializeOptions| {
-        let mut buffer = ryu::Buffer::new();
+        let mut buffer = zmij::Buffer::new();
         let value = buffer.format(item).as_bytes();
 
         for ch in value {
@@ -201,13 +176,7 @@ fn float_serializer_no_precision_autoformat_decimal_comma_<
     array: &'a PrimitiveArray<I>,
     f: F,
 ) -> impl Serializer<'a> {
-    make_serializer::<_, _, false>(f, array.iter(), |array| {
-        array
-            .as_any()
-            .downcast_ref::<PrimitiveArray<I>>()
-            .expect(ARRAY_MISMATCH_MSG)
-            .iter()
-    })
+    make_serializer::<_, _, false>(f, array.iter())
 }
 
 fn float_serializer_no_precision_scientific<I: NativeType + LowerExp>(
@@ -218,13 +187,7 @@ fn float_serializer_no_precision_scientific<I: NativeType + LowerExp>(
         let _ = write!(buf, "{item:.e}");
     };
 
-    make_serializer::<_, _, false>(f, array.iter(), |array| {
-        array
-            .as_any()
-            .downcast_ref::<PrimitiveArray<I>>()
-            .expect(ARRAY_MISMATCH_MSG)
-            .iter()
-    })
+    make_serializer::<_, _, false>(f, array.iter())
 }
 
 fn float_serializer_no_precision_scientific_decimal_comma<I: NativeType + LowerExp>(
@@ -244,13 +207,7 @@ fn float_serializer_no_precision_scientific_decimal_comma<I: NativeType + LowerE
         buf.extend_from_slice(&scratch);
     };
 
-    make_serializer::<_, _, false>(f, array.iter(), |array| {
-        array
-            .as_any()
-            .downcast_ref::<PrimitiveArray<I>>()
-            .expect(ARRAY_MISMATCH_MSG)
-            .iter()
-    })
+    make_serializer::<_, _, false>(f, array.iter())
 }
 
 fn float_serializer_no_precision_positional<I: NativeType + NumCast>(
@@ -261,13 +218,7 @@ fn float_serializer_no_precision_positional<I: NativeType + NumCast>(
         let _ = write!(buf, "{v}");
     };
 
-    make_serializer::<_, _, false>(f, array.iter(), |array| {
-        array
-            .as_any()
-            .downcast_ref::<PrimitiveArray<I>>()
-            .expect(ARRAY_MISMATCH_MSG)
-            .iter()
-    })
+    make_serializer::<_, _, false>(f, array.iter())
 }
 
 fn float_serializer_no_precision_positional_decimal_comma<I: NativeType + NumCast>(
@@ -288,13 +239,7 @@ fn float_serializer_no_precision_positional_decimal_comma<I: NativeType + NumCas
         buf.extend_from_slice(&scratch);
     };
 
-    make_serializer::<_, _, false>(f, array.iter(), |array| {
-        array
-            .as_any()
-            .downcast_ref::<PrimitiveArray<I>>()
-            .expect(ARRAY_MISMATCH_MSG)
-            .iter()
-    })
+    make_serializer::<_, _, false>(f, array.iter())
 }
 
 fn float_serializer_with_precision_scientific<I: NativeType + LowerExp>(
@@ -306,13 +251,7 @@ fn float_serializer_with_precision_scientific<I: NativeType + LowerExp>(
         let _ = write!(buf, "{item:.precision$e}");
     };
 
-    make_serializer::<_, _, false>(f, array.iter(), |array| {
-        array
-            .as_any()
-            .downcast_ref::<PrimitiveArray<I>>()
-            .expect(ARRAY_MISMATCH_MSG)
-            .iter()
-    })
+    make_serializer::<_, _, false>(f, array.iter())
 }
 
 fn float_serializer_with_precision_scientific_decimal_comma<I: NativeType + LowerExp>(
@@ -334,13 +273,7 @@ fn float_serializer_with_precision_scientific_decimal_comma<I: NativeType + Lowe
         buf.extend_from_slice(&scratch);
     };
 
-    make_serializer::<_, _, false>(f, array.iter(), |array| {
-        array
-            .as_any()
-            .downcast_ref::<PrimitiveArray<I>>()
-            .expect(ARRAY_MISMATCH_MSG)
-            .iter()
-    })
+    make_serializer::<_, _, false>(f, array.iter())
 }
 
 fn float_serializer_with_precision_positional<I: NativeType>(
@@ -352,13 +285,7 @@ fn float_serializer_with_precision_positional<I: NativeType>(
         let _ = write!(buf, "{item:.precision$}");
     };
 
-    make_serializer::<_, _, false>(f, array.iter(), |array| {
-        array
-            .as_any()
-            .downcast_ref::<PrimitiveArray<I>>()
-            .expect(ARRAY_MISMATCH_MSG)
-            .iter()
-    })
+    make_serializer::<_, _, false>(f, array.iter())
 }
 
 fn float_serializer_with_precision_positional_decimal_comma<I: NativeType>(
@@ -379,13 +306,7 @@ fn float_serializer_with_precision_positional_decimal_comma<I: NativeType>(
         buf.extend_from_slice(&scratch);
     };
 
-    make_serializer::<_, _, false>(f, array.iter(), |array| {
-        array
-            .as_any()
-            .downcast_ref::<PrimitiveArray<I>>()
-            .expect(ARRAY_MISMATCH_MSG)
-            .iter()
-    })
+    make_serializer::<_, _, false>(f, array.iter())
 }
 
 fn null_serializer(_array: &NullArray) -> impl Serializer<'_> {
@@ -394,7 +315,6 @@ fn null_serializer(_array: &NullArray) -> impl Serializer<'_> {
         fn serialize(&mut self, buf: &mut Vec<u8>, options: &SerializeOptions) {
             buf.extend_from_slice(options.null.as_bytes());
         }
-        fn update_array(&mut self, _array: &'a dyn Array) {}
     }
     NullSerializer
 }
@@ -405,13 +325,7 @@ fn bool_serializer<const QUOTE_NON_NULL: bool>(array: &BooleanArray) -> impl Ser
         buf.extend_from_slice(s.as_bytes());
     };
 
-    make_serializer::<_, _, QUOTE_NON_NULL>(f, array.iter(), |array| {
-        array
-            .as_any()
-            .downcast_ref::<BooleanArray>()
-            .expect(ARRAY_MISMATCH_MSG)
-            .iter()
-    })
+    make_serializer::<_, _, QUOTE_NON_NULL>(f, array.iter())
 }
 
 #[cfg(feature = "dtype-decimal")]
@@ -427,13 +341,7 @@ fn decimal_serializer(array: &PrimitiveArray<i128>, scale: usize) -> impl Serial
         );
     };
 
-    make_serializer::<_, _, false>(f, array.iter(), |array| {
-        array
-            .as_any()
-            .downcast_ref::<PrimitiveArray<i128>>()
-            .expect(ARRAY_MISMATCH_MSG)
-            .iter()
-    })
+    make_serializer::<_, _, false>(f, array.iter())
 }
 
 #[cfg(any(
@@ -449,13 +357,7 @@ fn callback_serializer<'a, T: NativeType, const QUOTE_NON_NULL: bool>(
         callback(item, buf);
     };
 
-    make_serializer::<_, _, QUOTE_NON_NULL>(f, array.iter(), |array| {
-        array
-            .as_any()
-            .downcast_ref::<PrimitiveArray<T>>()
-            .expect(ARRAY_MISMATCH_MSG)
-            .iter()
-    })
+    make_serializer::<_, _, QUOTE_NON_NULL>(f, array.iter())
 }
 
 #[cfg(any(feature = "dtype-date", feature = "dtype-time"))]
@@ -463,7 +365,7 @@ type ChronoFormatIter<'a, 'b> = std::slice::Iter<'a, chrono::format::Item<'b>>;
 
 #[cfg(any(feature = "dtype-date", feature = "dtype-time"))]
 fn date_and_time_serializer<'a, Underlying: NativeType, T: std::fmt::Display>(
-    format_str: &'a Option<String>,
+    format_str: Option<&'a str>,
     description: &str,
     array: &'a dyn Array,
     sample_value: T,
@@ -535,24 +437,18 @@ pub(super) fn string_serializer<'a, Iter: Send + 'a>(
     const LF: u8 = b'\n';
     const CR: u8 = b'\r';
 
-    struct StringSerializer<F, Iter, Update> {
+    struct StringSerializer<F, Iter> {
         serialize: F,
-        update: Update,
         iter: Iter,
     }
 
-    impl<'a, F, Iter, Update> Serializer<'a> for StringSerializer<F, Iter, Update>
+    impl<'a, F, Iter> Serializer<'a> for StringSerializer<F, Iter>
     where
         F: FnMut(&mut Iter, &mut Vec<u8>, &SerializeOptions),
-        Update: FnMut(&'a dyn Array) -> Iter,
     {
         fn serialize(&mut self, buf: &mut Vec<u8>, options: &SerializeOptions) {
             (self.serialize)(&mut self.iter, buf, options);
         }
-
-        fn update_array(&mut self, array: &'a dyn Array) {
-            self.iter = (self.update)(array);
-        }
     }
 
     fn serialize_str_escaped(buf: &mut Vec<u8>, s: &[u8], quote_char: u8, quoted: bool) {
@@ -601,11 +497,7 @@ pub(super) fn string_serializer<'a, Iter: Send + 'a>(
                     serialize_str_escaped(buf, s.as_bytes(), quote_char, true);
                     buf.push(quote_char);
                 };
-            Box::new(StringSerializer {
-                serialize,
-                update,
-                iter,
-            })
+            Box::new(StringSerializer { serialize, iter })
         },
         QuoteStyle::NonNumeric => {
             let serialize =
@@ -619,11 +511,7 @@ pub(super) fn string_serializer<'a, Iter: Send + 'a>(
                     serialize_str_escaped(buf, s.as_bytes(), quote_char, true);
                     buf.push(quote_char);
                 };
-            Box::new(StringSerializer {
-                serialize,
-                update,
-                iter,
-            })
+            Box::new(StringSerializer { serialize, iter })
         },
         QuoteStyle::Necessary => {
             let serialize =
@@ -647,11 +535,7 @@ pub(super) fn string_serializer<'a, Iter: Send + 'a>(
                         buf.push(quote_char);
                     }
                 };
-            Box::new(StringSerializer {
-                serialize,
-                update,
-                iter,
-            })
+            Box::new(StringSerializer { serialize, iter })
         },
         QuoteStyle::Never => {
             let serialize =
@@ -662,11 +546,7 @@ pub(super) fn string_serializer<'a, Iter: Send + 'a>(
                     };
                     buf.extend_from_slice(s.as_bytes());
                 };
-            Box::new(StringSerializer {
-                serialize,
-                update,
-                iter,
-            })
+            Box::new(StringSerializer { serialize, iter })
         },
     }
 }
@@ -679,10 +559,6 @@ fn quote_serializer<'a>(serializer: impl Serializer<'a>) -> impl Serializer<'a>
             self.0.serialize(buf, options);
             buf.push(options.quote_char);
         }
-
-        fn update_array(&mut self, array: &'a dyn Array) {
-            self.0.update_array(array);
-        }
     }
     QuoteSerializer(serializer)
 }
@@ -903,7 +779,7 @@ pub(super) fn serializer_for<'a>(
         },
         #[cfg(feature = "dtype-date")]
         DataType::Date => date_and_time_serializer(
-            &options.date_format,
+            options.date_format.as_deref(),
             "NaiveDate",
             array,
             chrono::NaiveDate::MAX,
@@ -913,7 +789,7 @@ pub(super) fn serializer_for<'a>(
         )?,
         #[cfg(feature = "dtype-time")]
         DataType::Time => date_and_time_serializer(
-            &options.time_format,
+            Some(options.time_format.as_deref().unwrap_or("%T%.9f")),
             "NaiveTime",
             array,
             chrono::NaiveTime::MIN,
