@@ -59,7 +59,6 @@ pub use expr::*;
 #[cfg(feature = "dtype-extension")]
 pub use extension::*;
 pub use function_expr::*;
-pub use functions::*;
 pub use list::*;
 pub use match_to_schema::*;
 #[cfg(feature = "meta")]
@@ -82,9 +81,9 @@ pub use struct_::*;
 pub use udf::UserDefinedFunction;
 mod file_scan;
 pub use file_scan::*;
+use functions::lit;
 pub use scan_sources::{ScanSource, ScanSourceIter, ScanSourceRef, ScanSources};
 
-pub use crate::plans::lit;
 use crate::prelude::*;
 
 impl Expr {
@@ -358,15 +357,17 @@ impl Expr {
             expr: Arc::new(self),
             idx: Arc::new(idx.into()),
             returns_scalar: false,
+            null_on_oob: false,
         }
     }
 
     /// Take the values by a single index.
-    pub fn get<E: Into<Expr>>(self, idx: E) -> Self {
+    pub fn get<E: Into<Expr>>(self, idx: E, null_on_oob: bool) -> Self {
         Expr::Gather {
             expr: Arc::new(self),
             idx: Arc::new(idx.into()),
             returns_scalar: true,
+            null_on_oob,
         }
     }
 
@@ -847,7 +848,7 @@ impl Expr {
             } else {
                 feature_gated!["dtype-struct", {
                     let e = e.iter().map(|e| e.clone().into()).collect::<Vec<_>>();
-                    Arc::new(as_struct(e))
+                    Arc::new(functions::as_struct(e))
                 }]
             };
             (e, options)
@@ -1718,26 +1719,3 @@ where
         fmt_str: Box::new(PlSmallStr::EMPTY),
     }
 }
-
-/// Return the number of rows in the context.
-pub fn len() -> Expr {
-    Expr::Len
-}
-
-/// First column in a DataFrame.
-pub fn first() -> Selector {
-    nth(0)
-}
-
-/// Last column in a DataFrame.
-pub fn last() -> Selector {
-    nth(-1)
-}
-
-/// Nth column in a DataFrame.
-pub fn nth(n: i64) -> Selector {
-    Selector::ByIndex {
-        indices: [n].into(),
-        strict: true,
-    }
-}
