@@ -10,7 +10,9 @@ use polars_utils::unique_id::UniqueId;
 use crate::dsl::{
     GroupbyOptions, HConcatOptions, JoinOptionsIR, JoinTypeOptionsIR, UnifiedScanArgs, UnionOptions,
 };
-use crate::plans::visualization::models::{Edge, IRNodeProperties};
+use crate::plans::visualization::models::{
+    Edge, IRNodeProperties, IntoWithArena, SortColumn, expr_list,
+};
 use crate::plans::{AExpr, ExprIR, FileInfo, IR};
 use crate::prelude::{DistinctOptionsIR, ProjectionOptions};
 
@@ -453,7 +455,7 @@ impl IRVisualizationDataGenerator<'_> {
                         row_estimation: _,
                     },
                 predicate,
-                predicate_file_skip_applied: _,
+                predicate_file_skip_applied,
                 scan_type,
                 unified_scan_args,
                 hive_parts,
@@ -510,6 +512,7 @@ impl IRVisualizationDataGenerator<'_> {
                     predicate: predicate
                         .as_ref()
                         .map(|e| format_pl_smallstr!("{}", e.display(self.expr_arena))),
+                    predicate_file_skip_applied: *predicate_file_skip_applied,
                     has_table_statistics: table_statistics.is_some(),
                     include_file_paths: include_file_paths.clone(),
                     column_mapping_type: column_mapping
@@ -570,7 +573,7 @@ impl IRVisualizationDataGenerator<'_> {
             },
             IR::Sink { input: _, payload } => {
                 let properties = IRNodeProperties::Sink {
-                    payload: format_pl_smallstr!("{:?}", payload),
+                    payload: payload.into_with_arena(self.expr_arena),
                 };
 
                 IRNodeInfo {
@@ -621,10 +624,17 @@ impl IRVisualizationDataGenerator<'_> {
                     },
             } => {
                 let properties = IRNodeProperties::Sort {
-                    by_exprs: expr_list(by_column, self.expr_arena),
+                    sort_columns: by_column
+                        .iter()
+                        .zip(descending.iter())
+                        .zip(nulls_last.iter())
+                        .map(|((expr, &descending), &nulls_last)| SortColumn {
+                            expr: format_pl_smallstr!("{}", expr.display(self.expr_arena)),
+                            descending,
+                            nulls_last,
+                        })
+                        .collect(),
                     slice: convert_opt_slice(slice),
-                    descending: descending.clone(),
-                    nulls_last: nulls_last.clone(),
                     multithreaded: *multithreaded,
                     maintain_order: *maintain_order,
                     #[cfg_attr(feature = "bigidx", expect(clippy::useless_conversion))]
@@ -747,10 +757,3 @@ where
 {
     slice.map(|(offset, len)| (offset.try_into().unwrap(), len.try_into().unwrap()))
 }
-
-fn expr_list(exprs: &[ExprIR], expr_arena: &Arena<AExpr>) -> Vec<PlSmallStr> {
-    exprs
-        .iter()
-        .map(|e| format_pl_smallstr!("{}", e.display(expr_arena)))
-        .collect()
-}
