@@ -20,7 +20,7 @@ impl IRNode {
             node: *self,
             lp_arena,
             expr_arena,
-            ignore_cache: false,
+            hash_as_equality: false,
         }
     }
 }
@@ -29,14 +29,12 @@ pub(crate) struct HashableEqLP<'a> {
     node: IRNode,
     lp_arena: &'a Arena<IR>,
     expr_arena: &'a Arena<AExpr>,
-    ignore_cache: bool,
+    hash_as_equality: bool,
 }
 
 impl HashableEqLP<'_> {
-    /// When encountering a Cache node, ignore it and take the input.
-    #[cfg(feature = "cse")]
-    pub(crate) fn ignore_caches(mut self) -> Self {
-        self.ignore_cache = true;
+    pub fn hash_as_equality(mut self) -> Self {
+        self.hash_as_equality = true;
         self
     }
 }
@@ -109,6 +107,9 @@ impl Hash for HashableEqLP<'_> {
             } => {
                 // Hash the Python function object using the pointer to the object
                 // This should be the same as calling id() in python, but we don't need the GIL
+
+                use std::sync::atomic::AtomicU64;
+                static UNIQUE_COUNT: AtomicU64 = AtomicU64::new(0);
                 if let Some(scan_fn) = scan_fn {
                     let ptr_addr = scan_fn.0.as_ptr() as usize;
                     ptr_addr.hash(state);
@@ -122,7 +123,13 @@ impl Hash for HashableEqLP<'_> {
                 n_rows.hash(state);
                 hash_python_predicate(predicate, self.expr_arena, state);
                 validate_schema.hash(state);
-                is_pure.hash(state);
+
+                if self.hash_as_equality && !*is_pure {
+                    let val = UNIQUE_COUNT.fetch_add(1, std::sync::atomic::Ordering::Relaxed);
+                    val.hash(state)
+                } else {
+                    is_pure.hash(state)
+                }
             },
             IR::Slice {
                 offset,
@@ -247,7 +254,7 @@ impl Hash for HashableEqLP<'_> {
             IR::Sink { input: _, payload } => {
                 payload.traverse_and_hash(self.expr_arena, state);
             },
-            IR::SinkMultiple { .. } => {},
+            IR::SinkMultiple { inputs: _ } => {},
             IR::Cache { input: _, id } => {
                 id.hash(state);
             },
@@ -600,27 +607,6 @@ impl PartialEq for HashableEqLP<'_> {
                     let l_alp = l.to_alp(self.lp_arena);
                     let r_alp = r.to_alp(self.lp_arena);
 
-                    if self.ignore_cache {
-                        match (l_alp, r_alp) {
-                            (IR::Cache { input: l, .. }, IR::Cache { input: r, .. }) => {
-                                scratch_1.push(*l);
-                                scratch_2.push(*r);
-                                continue;
-                            },
-                            (IR::Cache { input: l, .. }, _) => {
-                                scratch_1.push(*l);
-                                scratch_2.push(r.node());
-                                continue;
-                            },
-                            (_, IR::Cache { input: r, .. }) => {
-                                scratch_1.push(l.node());
-                                scratch_2.push(*r);
-                                continue;
-                            },
-                            _ => {},
-                        }
-                    }
-
                     if !l
                         .hashable_and_cmp(self.lp_arena, self.expr_arena)
                         .is_equal(&r.hashable_and_cmp(self.lp_arena, self.expr_arena))
