@@ -27,7 +27,7 @@ impl PyDataFrame {
         let columns = columns.to_series();
         // @scalar-opt
         let columns = columns.into_iter().map(|s| s.into()).collect();
-        let df = DataFrame::new(columns).map_err(PyPolarsErr::from)?;
+        let df = DataFrame::new_infer_height(columns).map_err(PyPolarsErr::from)?;
         Ok(PyDataFrame::new(df))
     }
 
@@ -38,7 +38,7 @@ impl PyDataFrame {
     pub fn dtype_strings(&self) -> Vec<String> {
         self.df
             .read()
-            .get_columns()
+            .columns()
             .iter()
             .map(|s| format!("{}", s.dtype()))
             .collect()
@@ -119,7 +119,7 @@ impl PyDataFrame {
     pub fn rechunk(&self, py: Python) -> PyResult<Self> {
         py.enter_polars_df(|| {
             let mut df = self.df.read().clone();
-            df.as_single_chunk_par();
+            df.rechunk_mut_par();
             Ok(df)
         })
     }
@@ -130,7 +130,7 @@ impl PyDataFrame {
     }
 
     pub fn get_columns(&self) -> Vec<PySeries> {
-        let cols = self.df.read().get_columns().to_vec();
+        let cols = self.df.read().columns().to_vec();
         cols.to_pyseries()
     }
 
@@ -138,7 +138,7 @@ impl PyDataFrame {
     pub fn columns(&self) -> Vec<String> {
         self.df
             .read()
-            .get_columns()
+            .columns()
             .iter()
             .map(|s| s.name().to_string())
             .collect()
@@ -148,7 +148,7 @@ impl PyDataFrame {
     pub fn set_column_names(&self, names: Vec<PyBackedStr>) -> PyResult<()> {
         self.df
             .write()
-            .set_column_names(names.iter().map(|x| &**x))
+            .set_column_names(&names)
             .map_err(PyPolarsErr::from)?;
         Ok(())
     }
@@ -157,6 +157,7 @@ impl PyDataFrame {
     pub fn dtypes<'py>(&self, py: Python<'py>) -> PyResult<Bound<'py, PyList>> {
         let df = self.df.read();
         let iter = df
+            .columns()
             .iter()
             .map(|s| Wrap(s.dtype().clone()).into_pyobject(py).unwrap());
         PyList::new(py, iter)
@@ -179,7 +180,7 @@ impl PyDataFrame {
     }
 
     pub fn is_empty(&self) -> bool {
-        self.df.read().is_empty()
+        self.df.read().shape_has_zero()
     }
 
     pub fn hstack(&self, py: Python<'_>, columns: Vec<PySeries>) -> PyResult<Self> {
@@ -205,7 +206,7 @@ impl PyDataFrame {
         py.enter_polars(|| {
             // Prevent self-vstack deadlocks.
             let other = other.df.read().clone();
-            self.df.write().vstack_mut(&other)?;
+            self.df.write().vstack_mut_owned(other)?;
             PolarsResult::Ok(())
         })?;
         Ok(())
@@ -285,7 +286,7 @@ impl PyDataFrame {
     pub fn replace(&self, column: &str, new_col: PySeries) -> PyResult<()> {
         self.df
             .write()
-            .replace(column, new_col.series.into_inner())
+            .replace(column, new_col.series.into_inner().into_column())
             .map_err(PyPolarsErr::from)?;
         Ok(())
     }
@@ -293,7 +294,7 @@ impl PyDataFrame {
     pub fn replace_column(&self, index: usize, new_column: PySeries) -> PyResult<()> {
         self.df
             .write()
-            .replace_column(index, new_column.series.into_inner())
+            .replace_column(index, new_column.series.into_inner().into_column())
             .map_err(PyPolarsErr::from)?;
         Ok(())
     }
@@ -301,7 +302,7 @@ impl PyDataFrame {
     pub fn insert_column(&self, index: usize, column: PySeries) -> PyResult<()> {
         self.df
             .write()
-            .insert_column(index, column.series.into_inner())
+            .insert_column(index, column.series.into_inner().into_column())
             .map_err(PyPolarsErr::from)?;
         Ok(())
     }
@@ -554,7 +555,7 @@ impl PyDataFrame {
         use polars_ffi::version_0::export_column;
 
         let df = self.df.read();
-        let cols = df.get_columns();
+        let cols = df.columns();
 
         let location = location as *mut SeriesExport;
 
@@ -593,16 +594,17 @@ impl PyDataFrame {
             let is_unordered = opts.first().is_some_and(|(_, _, v)| *v);
 
             let ca = if is_unordered {
-                _get_rows_encoded_ca_unordered(name, self.df.read().get_columns())
+                _get_rows_encoded_ca_unordered(name, self.df.read().columns())
             } else {
                 let descending = opts.iter().map(|(v, _, _)| *v).collect::<Vec<_>>();
                 let nulls_last = opts.iter().map(|(_, v, _)| *v).collect::<Vec<_>>();
 
                 _get_rows_encoded_ca(
                     name,
-                    self.df.read().get_columns(),
+                    self.df.read().columns(),
                     descending.as_slice(),
                     nulls_last.as_slice(),
+                    false,
                 )
             }?;
 
