@@ -1,11 +1,11 @@
 use std::sync::Arc;
 
 use parking_lot::Mutex;
-use polars_core::SchemaExtPl;
 use polars_core::frame::{DataFrame, UniqueKeepStrategy};
 use polars_core::prelude::{DataType, PlHashMap, PlHashSet};
 use polars_core::scalar::Scalar;
 use polars_core::schema::Schema;
+use polars_core::{SchemaExtPl, config};
 use polars_error::{PolarsResult, polars_bail};
 use polars_expr::state::ExecutionState;
 use polars_mem_engine::create_physical_plan;
@@ -14,8 +14,8 @@ use polars_plan::dsl::default_values::DefaultFieldValues;
 use polars_plan::dsl::deletion::DeletionFilesList;
 use polars_plan::dsl::sink2::FileProviderType;
 use polars_plan::dsl::{
-    CallbackSinkType, ExtraColumnsPolicy, FileScanIR, FileSinkOptions, PartitionStrategyIR,
-    PartitionVariantIR, PartitionedSinkOptionsIR, SinkOptions, SinkTypeIR, UnifiedSinkArgs,
+    CallbackSinkType, ExtraColumnsPolicy, FileScanIR, PartitionStrategyIR, PartitionVariantIR,
+    PartitionedSinkOptionsIR, SinkOptions, SinkTypeIR, UnifiedSinkArgs,
 };
 use polars_plan::plans::expr_ir::{ExprIR, OutputName};
 use polars_plan::plans::{
@@ -26,6 +26,9 @@ use polars_plan::prelude::GroupbyOptions;
 use polars_utils::arena::{Arena, Node};
 use polars_utils::itertools::Itertools;
 use polars_utils::pl_str::PlSmallStr;
+#[cfg(feature = "parquet")]
+use polars_utils::relaxed_cell::RelaxedCell;
+use polars_utils::row_counter::RowCounter;
 use polars_utils::slice_enum::Slice;
 use polars_utils::unique_id::UniqueId;
 use polars_utils::{IdxSize, format_pl_smallstr, unique_column_name};
@@ -265,63 +268,37 @@ pub fn lower_ir(
                 }
             },
 
-            SinkTypeIR::File(options)
-                if match options.file_format.as_ref() {
-                    #[cfg(feature = "parquet")]
-                    polars_plan::dsl::FileType::Parquet(_) => true,
-                    _ => false,
-                } =>
-            {
+            SinkTypeIR::File(options) => {
                 let options = options.clone();
                 let input = lower_ir!(*input)?;
-                PhysNodeKind::FileSink2 { input, options }
+                PhysNodeKind::FileSink { input, options }
             },
 
             SinkTypeIR::Partitioned(options)
                 if !matches!(options.file_path_provider, FileProviderType::Legacy(_))
                     && options.finish_callback.is_none()
-                    && match options.file_format.as_ref() {
+                    && match options.file_format {
                         #[cfg(feature = "parquet")]
-                        polars_plan::dsl::FileType::Parquet(_) => true,
-                        _ => false,
+                        polars_plan::dsl::FileWriteFormat::Parquet(_) => true,
+                        #[cfg(feature = "ipc")]
+                        polars_plan::dsl::FileWriteFormat::Ipc(_) => true,
+                        #[cfg(feature = "csv")]
+                        polars_plan::dsl::FileWriteFormat::Csv(_) => true,
+                        #[cfg(feature = "json")]
+                        polars_plan::dsl::FileWriteFormat::NDJson(_) => true,
+                        #[cfg(not(any(
+                            feature = "parquet",
+                            feature = "ipc",
+                            feature = "csv",
+                            feature = "json"
+                        )))]
+                        _ => panic!("no enum variants on FileType (hint: missing feature flags?)"),
                     } =>
             {
                 let options = options.clone();
                 let input = lower_ir!(*input)?;
                 PhysNodeKind::PartitionedSink2 { input, options }
             },
-            SinkTypeIR::File(FileSinkOptions {
-                target,
-                file_format,
-                unified_sink_args,
-            }) => {
-                // Convert to old parameters for now
-
-                let target = target.clone();
-
-                let UnifiedSinkArgs {
-                    mkdir,
-                    maintain_order,
-                    sync_on_close,
-                    cloud_options,
-                } = unified_sink_args.clone();
-
-                let sink_options = SinkOptions {
-                    sync_on_close,
-                    maintain_order,
-                    mkdir,
-                };
-                let file_type = file_format.as_ref().clone();
-
-                let phys_input = lower_ir!(*input)?;
-                PhysNodeKind::FileSink {
-                    target,
-                    sink_options,
-                    file_type,
-                    input: phys_input,
-                    cloud_options: cloud_options.map(Arc::unwrap_or_clone),
-                }
-            },
 
             SinkTypeIR::Partitioned(PartitionedSinkOptionsIR {
                 base_path,
@@ -341,7 +318,7 @@ pub fn lower_ir(
                     FileProviderType::Hive { .. } => None,
                     FileProviderType::Function(_) => None,
                 };
-                let file_type = file_format.as_ref().clone();
+                let file_type = file_format.clone();
                 let finish_callback = finish_callback.clone();
 
                 let (variant, per_partition_sort_by) = match partition_strategy {
@@ -484,6 +461,7 @@ pub fn lower_ir(
                         RowEncodingVariant::Ordered {
                             descending: None,
                             nulls_last: None,
+                            broadcast_nulls: None,
                         },
                         key_dtype,
                         expr_arena,
@@ -717,271 +695,308 @@ pub fn lower_ir(
                 unreachable!();
             };
 
-            let file_reader_builder: Arc<dyn FileReaderBuilder> = match &*scan_type {
-                #[cfg(feature = "parquet")]
-                FileScanIR::Parquet {
-                    options,
-                    metadata: first_metadata,
-                } => Arc::new(
-                    crate::nodes::io_sources::parquet::builder::ParquetReaderBuilder {
-                        options: Arc::new(options.clone()),
-                        first_metadata: first_metadata.clone(),
-                    },
-                ) as _,
-
-                #[cfg(feature = "ipc")]
-                FileScanIR::Ipc {
-                    options: polars_io::ipc::IpcScanOptions {},
-                    metadata: first_metadata,
-                } => Arc::new(crate::nodes::io_sources::ipc::builder::IpcReaderBuilder {
-                    first_metadata: first_metadata.clone(),
-                }) as _,
-
-                #[cfg(feature = "csv")]
-                FileScanIR::Csv { options } => Arc::new(Arc::new(options.clone())) as _,
-
-                #[cfg(feature = "json")]
-                FileScanIR::NDJson { options } => Arc::new(Arc::new(options.clone())) as _,
-
-                #[cfg(feature = "python")]
-                FileScanIR::PythonDataset {
-                    dataset_object: _,
-                    cached_ir,
-                } => {
-                    use crate::physical_plan::io::python_dataset::python_dataset_scan_to_reader_builder;
-                    let guard = cached_ir.lock().unwrap();
-
-                    let expanded_scan = guard
-                        .as_ref()
-                        .expect("python dataset should be resolved")
-                        .python_scan()
-                        .expect("should be python scan");
-
-                    python_dataset_scan_to_reader_builder(expanded_scan)
-                },
-
-                #[cfg(feature = "scan_lines")]
-                FileScanIR::Lines { name: _ } => todo!(),
-
-                FileScanIR::Anonymous { .. } => todo!("unimplemented: AnonymousScan"),
-            };
-
+            if (scan_sources.is_empty()
+                && !matches!(scan_type.as_ref(), FileScanIR::Anonymous { .. }))
+                || unified_scan_args
+                    .pre_slice
+                    .as_ref()
+                    .is_some_and(|slice| slice.len() == 0)
             {
-                let cloud_options = &unified_scan_args.cloud_options;
-                let output_schema =
-                    if std::env::var("POLARS_FORCE_EMPTY_PROJECT").as_deref() == Ok("1") {
-                        Default::default()
-                    } else {
-                        output_schema
-                    };
+                if config::verbose() {
+                    eprintln!("lower_ir: scan IR lowered as empty InMemorySource")
+                }
 
-                let cloud_options = cloud_options.clone().map(Arc::new);
-                let file_schema = file_info.schema;
+                // If there are no sources, just provide an empty in-memory source with the right
+                // schema.
+                PhysNodeKind::InMemorySource {
+                    df: Arc::new(DataFrame::empty_with_schema(output_schema.as_ref())),
+                }
+            } else if output_schema.is_empty()
+                && let Some((physical_rows, deleted_rows)) = unified_scan_args.row_count
+                && unified_scan_args.pre_slice.is_none()
+                && predicate.is_none()
+            {
+                // Fast-count for scan_iceberg will hit here.
+                let row_counter = RowCounter::new(physical_rows, deleted_rows);
+                row_counter.num_rows_idxsize()?;
+                let num_rows = row_counter.num_rows()?;
 
-                let (projected_schema, file_schema) =
-                    multi_scan::functions::resolve_projections::resolve_projections(
-                        &output_schema,
-                        &file_schema,
-                        &mut hive_parts,
-                        unified_scan_args
-                            .row_index
+                if config::verbose() {
+                    eprintln!(
+                        "lower_ir: scan IR lowered as 0-width InMemorySource with height {} ({:?})",
+                        num_rows, &row_counter
+                    )
+                }
+
+                PhysNodeKind::InMemorySource {
+                    df: Arc::new(DataFrame::empty_with_height(num_rows)),
+                }
+            } else {
+                let file_reader_builder: Arc<dyn FileReaderBuilder> = match &*scan_type {
+                    #[cfg(feature = "parquet")]
+                    FileScanIR::Parquet {
+                        options,
+                        metadata: first_metadata,
+                    } => Arc::new(
+                        crate::nodes::io_sources::parquet::builder::ParquetReaderBuilder {
+                            options: Arc::new(options.clone()),
+                            first_metadata: first_metadata.clone(),
+                            prefetch_limit: RelaxedCell::new_usize(0),
+                            prefetch_semaphore: std::sync::OnceLock::new(),
+                            shared_prefetch_wait_group_slot: Default::default(),
+                        },
+                    ) as _,
+
+                    #[cfg(feature = "ipc")]
+                    FileScanIR::Ipc {
+                        options: polars_io::ipc::IpcScanOptions {},
+                        metadata: first_metadata,
+                    } => Arc::new(crate::nodes::io_sources::ipc::builder::IpcReaderBuilder {
+                        first_metadata: first_metadata.clone(),
+                        prefetch_limit: RelaxedCell::new_usize(0),
+                        prefetch_semaphore: std::sync::OnceLock::new(),
+                        shared_prefetch_wait_group_slot: Default::default(),
+                    }) as _,
+
+                    #[cfg(feature = "csv")]
+                    FileScanIR::Csv { options } => Arc::new(Arc::clone(options)) as _,
+
+                    #[cfg(feature = "json")]
+                    FileScanIR::NDJson { options } => Arc::new(Arc::new(options.clone())) as _,
+
+                    #[cfg(feature = "python")]
+                    FileScanIR::PythonDataset {
+                        dataset_object: _,
+                        cached_ir,
+                    } => {
+                        use crate::physical_plan::io::python_dataset::python_dataset_scan_to_reader_builder;
+                        let guard = cached_ir.lock().unwrap();
+
+                        let expanded_scan = guard
                             .as_ref()
-                            .map(|ri| ri.name.as_str()),
+                            .expect("python dataset should be resolved")
+                            .python_scan()
+                            .expect("should be python scan");
+
+                        python_dataset_scan_to_reader_builder(expanded_scan)
+                    },
+
+                    #[cfg(feature = "scan_lines")]
+                    FileScanIR::Lines { name: _ } => todo!(),
+
+                    FileScanIR::Anonymous { .. } => todo!("unimplemented: AnonymousScan"),
+                };
+
+                {
+                    let cloud_options = unified_scan_args.cloud_options.clone().map(Arc::new);
+                    let file_schema = file_info.schema;
+
+                    let (projected_schema, file_schema) =
+                        multi_scan::functions::resolve_projections::resolve_projections(
+                            &output_schema,
+                            &file_schema,
+                            &mut hive_parts,
+                            unified_scan_args
+                                .row_index
+                                .as_ref()
+                                .map(|ri| ri.name.as_str()),
+                            unified_scan_args
+                                .include_file_paths
+                                .as_ref()
+                                .map(|x| x.as_str()),
+                        );
+
+                    let file_projection_builder = ProjectionBuilder::new(
+                        projected_schema,
+                        unified_scan_args.column_mapping.as_ref(),
                         unified_scan_args
-                            .include_file_paths
-                            .as_ref()
-                            .map(|x| x.as_str()),
+                            .default_values
+                            .filter(|DefaultFieldValues::Iceberg(v)| !v.is_empty())
+                            .map(|DefaultFieldValues::Iceberg(v)| v),
                     );
 
-                let file_projection_builder = ProjectionBuilder::new(
-                    projected_schema,
-                    unified_scan_args.column_mapping.as_ref(),
-                    unified_scan_args
-                        .default_values
-                        .filter(|DefaultFieldValues::Iceberg(v)| !v.is_empty())
-                        .map(|DefaultFieldValues::Iceberg(v)| v),
-                );
+                    // TODO: We ignore the parameter for some scan types to maintain old behavior,
+                    // as they currently don't expose an API for it to be configured.
+                    let extra_columns_policy = match &*scan_type {
+                        #[cfg(feature = "parquet")]
+                        FileScanIR::Parquet { .. } => unified_scan_args.extra_columns_policy,
 
-                // TODO: We ignore the parameter for some scan types to maintain old behavior,
-                // as they currently don't expose an API for it to be configured.
-                let extra_columns_policy = match &*scan_type {
-                    #[cfg(feature = "parquet")]
-                    FileScanIR::Parquet { .. } => unified_scan_args.extra_columns_policy,
+                        _ => {
+                            if unified_scan_args.projection.is_some() {
+                                ExtraColumnsPolicy::Ignore
+                            } else {
+                                ExtraColumnsPolicy::Raise
+                            }
+                        },
+                    };
 
-                    _ => {
-                        if unified_scan_args.projection.is_some() {
-                            ExtraColumnsPolicy::Ignore
-                        } else {
-                            ExtraColumnsPolicy::Raise
+                    let forbid_extra_columns = ForbidExtraColumns::opt_new(
+                        &extra_columns_policy,
+                        &file_schema,
+                        unified_scan_args.column_mapping.as_ref(),
+                    );
+
+                    let pre_slice = unified_scan_args.pre_slice.clone();
+
+                    let mut multi_scan_node = PhysNodeKind::MultiScan {
+                        scan_sources,
+                        file_reader_builder,
+                        cloud_options,
+                        file_projection_builder,
+                        output_schema: output_schema.clone(),
+                        row_index: None,
+                        pre_slice,
+                        predicate,
+                        predicate_file_skip_applied,
+                        hive_parts,
+                        cast_columns_policy: unified_scan_args.cast_columns_policy,
+                        missing_columns_policy: unified_scan_args.missing_columns_policy,
+                        forbid_extra_columns,
+                        include_file_paths: unified_scan_args.include_file_paths,
+                        // Set to None if empty for performance.
+                        deletion_files: DeletionFilesList::filter_empty(
+                            unified_scan_args.deletion_files,
+                        ),
+                        table_statistics: unified_scan_args.table_statistics,
+                        file_schema,
+                    };
+
+                    let PhysNodeKind::MultiScan {
+                        output_schema: multi_scan_output_schema,
+                        row_index: row_index_to_multiscan,
+                        pre_slice: pre_slice_to_multiscan,
+                        predicate: predicate_to_multiscan,
+                        ..
+                    } = &mut multi_scan_node
+                    else {
+                        unreachable!()
+                    };
+
+                    let mut row_index_post = unified_scan_args.row_index;
+
+                    // * If a predicate was pushed then we always push row index
+                    if predicate_to_multiscan.is_some()
+                        || matches!(pre_slice_to_multiscan, Some(Slice::Negative { .. }))
+                    {
+                        *row_index_to_multiscan = row_index_post.take();
+                    }
+
+                    // TODO
+                    // Projection pushdown could change the row index column position. Ideally it shouldn't,
+                    // and instead just put a projection on top of the scan node in the IR. But for now
+                    // we do that step here.
+                    let mut schema_after_row_index_post = multi_scan_output_schema.clone();
+                    let mut reorder_after_row_index_post = false;
+
+                    // Remove row index from multiscan schema if not pushed.
+                    if let Some(ri) = row_index_post.as_ref() {
+                        let row_index_post_position =
+                            multi_scan_output_schema.index_of(&ri.name).unwrap();
+                        let (_, dtype) = Arc::make_mut(multi_scan_output_schema)
+                            .shift_remove_index(row_index_post_position)
+                            .unwrap();
+
+                        if row_index_post_position != 0 {
+                            reorder_after_row_index_post = true;
+                            let mut schema =
+                                Schema::with_capacity(multi_scan_output_schema.len() + 1);
+                            schema.extend([(ri.name.clone(), dtype)]);
+                            schema.extend(
+                                multi_scan_output_schema
+                                    .iter()
+                                    .map(|(k, v)| (k.clone(), v.clone())),
+                            );
+                            schema_after_row_index_post = Arc::new(schema);
                         }
-                    },
-                };
+                    }
 
-                let forbid_extra_columns = ForbidExtraColumns::opt_new(
-                    &extra_columns_policy,
-                    &file_schema,
-                    unified_scan_args.column_mapping.as_ref(),
-                );
+                    // If we have no predicate and no slice or positive slice, we can reorder the row index to after
+                    // the slice by adjusting the offset. This can remove a serial synchronization step in multiscan
+                    // and allow the reader to still skip rows.
+                    let row_index_post_after_slice = (|| {
+                        let mut row_index = row_index_post.take()?;
 
-                let pre_slice = unified_scan_args.pre_slice.clone();
+                        let positive_offset = match pre_slice_to_multiscan {
+                            Some(Slice::Positive { offset, .. }) => Some(*offset),
+                            None => Some(0),
+                            Some(Slice::Negative { .. }) => unreachable!(),
+                        }?;
 
-                let mut multi_scan_node = PhysNodeKind::MultiScan {
-                    scan_sources,
-                    file_reader_builder,
-                    cloud_options,
-                    file_projection_builder,
-                    output_schema: output_schema.clone(),
-                    row_index: None,
-                    pre_slice,
-                    predicate,
-                    predicate_file_skip_applied,
-                    hive_parts,
-                    cast_columns_policy: unified_scan_args.cast_columns_policy,
-                    missing_columns_policy: unified_scan_args.missing_columns_policy,
-                    forbid_extra_columns,
-                    include_file_paths: unified_scan_args.include_file_paths,
-                    // Set to None if empty for performance.
-                    deletion_files: DeletionFilesList::filter_empty(
-                        unified_scan_args.deletion_files,
-                    ),
-                    table_statistics: unified_scan_args.table_statistics,
-                    file_schema,
-                };
-
-                let PhysNodeKind::MultiScan {
-                    output_schema: multi_scan_output_schema,
-                    row_index: row_index_to_multiscan,
-                    pre_slice: pre_slice_to_multiscan,
-                    predicate: predicate_to_multiscan,
-                    ..
-                } = &mut multi_scan_node
-                else {
-                    unreachable!()
-                };
-
-                let mut row_index_post = unified_scan_args.row_index;
-
-                // * If a predicate was pushed then we always push row index
-                if predicate_to_multiscan.is_some()
-                    || matches!(pre_slice_to_multiscan, Some(Slice::Negative { .. }))
-                {
-                    *row_index_to_multiscan = row_index_post.take();
-                }
-
-                // TODO
-                // Projection pushdown could change the row index column position. Ideally it shouldn't,
-                // and instead just put a projection on top of the scan node in the IR. But for now
-                // we do that step here.
-                let mut schema_after_row_index_post = multi_scan_output_schema.clone();
-                let mut reorder_after_row_index_post = false;
-
-                // Remove row index from multiscan schema if not pushed.
-                if let Some(ri) = row_index_post.as_ref() {
-                    let row_index_post_position =
-                        multi_scan_output_schema.index_of(&ri.name).unwrap();
-                    let (_, dtype) = Arc::make_mut(multi_scan_output_schema)
-                        .shift_remove_index(row_index_post_position)
-                        .unwrap();
-
-                    if row_index_post_position != 0 {
-                        reorder_after_row_index_post = true;
-                        let mut schema = Schema::with_capacity(multi_scan_output_schema.len() + 1);
-                        schema.extend([(ri.name.clone(), dtype)]);
-                        schema.extend(
-                            multi_scan_output_schema
-                                .iter()
-                                .map(|(k, v)| (k.clone(), v.clone())),
+                        row_index.offset = row_index.offset.saturating_add(
+                            IdxSize::try_from(positive_offset).unwrap_or(IdxSize::MAX),
                         );
-                        schema_after_row_index_post = Arc::new(schema);
-                    }
-                }
 
-                // If we have no predicate and no slice or positive slice, we can reorder the row index to after
-                // the slice by adjusting the offset. This can remove a serial synchronization step in multiscan
-                // and allow the reader to still skip rows.
-                let row_index_post_after_slice = (|| {
-                    let mut row_index = row_index_post.take()?;
+                        Some(row_index)
+                    })();
 
-                    let positive_offset = match pre_slice_to_multiscan {
-                        Some(Slice::Positive { offset, .. }) => Some(*offset),
-                        None => Some(0),
-                        Some(Slice::Negative { .. }) => unreachable!(),
-                    }?;
-
-                    row_index.offset = row_index
-                        .offset
-                        .saturating_add(IdxSize::try_from(positive_offset).unwrap_or(IdxSize::MAX));
-
-                    Some(row_index)
-                })();
-
-                let mut stream = {
-                    let node_key = phys_sm.insert(PhysNode::new(
-                        multi_scan_output_schema.clone(),
-                        multi_scan_node,
-                    ));
-                    PhysStream::first(node_key)
-                };
-
-                if let Some(ri) = row_index_post {
-                    let node = PhysNodeKind::WithRowIndex {
-                        input: stream,
-                        name: ri.name,
-                        offset: Some(ri.offset),
+                    let mut stream = {
+                        let node_key = phys_sm.insert(PhysNode::new(
+                            multi_scan_output_schema.clone(),
+                            multi_scan_node,
+                        ));
+                        PhysStream::first(node_key)
                     };
 
-                    let node_key = phys_sm.insert(PhysNode {
-                        output_schema: schema_after_row_index_post.clone(),
-                        kind: node,
-                    });
-
-                    stream = PhysStream::first(node_key);
-
-                    if reorder_after_row_index_post {
-                        let node = PhysNodeKind::SimpleProjection {
+                    if let Some(ri) = row_index_post {
+                        let node = PhysNodeKind::WithRowIndex {
                             input: stream,
-                            columns: output_schema.iter_names_cloned().collect(),
+                            name: ri.name,
+                            offset: Some(ri.offset),
                         };
 
                         let node_key = phys_sm.insert(PhysNode {
-                            output_schema: output_schema.clone(),
+                            output_schema: schema_after_row_index_post.clone(),
                             kind: node,
                         });
 
                         stream = PhysStream::first(node_key);
+
+                        if reorder_after_row_index_post {
+                            let node = PhysNodeKind::SimpleProjection {
+                                input: stream,
+                                columns: output_schema.iter_names_cloned().collect(),
+                            };
+
+                            let node_key = phys_sm.insert(PhysNode {
+                                output_schema: output_schema.clone(),
+                                kind: node,
+                            });
+
+                            stream = PhysStream::first(node_key);
+                        }
                     }
-                }
 
-                if let Some(ri) = row_index_post_after_slice {
-                    let node = PhysNodeKind::WithRowIndex {
-                        input: stream,
-                        name: ri.name,
-                        offset: Some(ri.offset),
-                    };
-
-                    let node_key = phys_sm.insert(PhysNode {
-                        output_schema: schema_after_row_index_post,
-                        kind: node,
-                    });
-
-                    stream = PhysStream::first(node_key);
-
-                    if reorder_after_row_index_post {
-                        let node = PhysNodeKind::SimpleProjection {
+                    if let Some(ri) = row_index_post_after_slice {
+                        let node = PhysNodeKind::WithRowIndex {
                             input: stream,
-                            columns: output_schema.iter_names_cloned().collect(),
+                            name: ri.name,
+                            offset: Some(ri.offset),
                         };
 
                         let node_key = phys_sm.insert(PhysNode {
-                            output_schema: output_schema.clone(),
+                            output_schema: schema_after_row_index_post,
                             kind: node,
                         });
 
                         stream = PhysStream::first(node_key);
-                    }
-                }
 
-                return Ok(stream);
+                        if reorder_after_row_index_post {
+                            let node = PhysNodeKind::SimpleProjection {
+                                input: stream,
+                                columns: output_schema.iter_names_cloned().collect(),
+                            };
+
+                            let node_key = phys_sm.insert(PhysNode {
+                                output_schema: output_schema.clone(),
+                                kind: node,
+                            });
+
+                            stream = PhysStream::first(node_key);
+                        }
+                    }
+
+                    return Ok(stream);
+                }
             }
         },
 
