@@ -3,6 +3,7 @@ use std::num::NonZeroUsize;
 use polars_io::utils::sync_on_close::SyncOnCloseType;
 use polars_ops::frame::MaintainOrderJoin;
 use polars_ops::prelude::{JoinCoalesce, JoinValidation};
+use polars_plan::dsl::PredicateFileSkip;
 use polars_plan::dsl::sink2::FileProviderType;
 use polars_utils::IdxSize;
 use polars_utils::pl_str::PlSmallStr;
@@ -95,8 +96,8 @@ pub enum PhysNodeProperties {
         offset: u64,
     },
     GroupBy {
-        keys: Vec<PlSmallStr>,
-        aggs: Vec<PlSmallStr>,
+        key_per_input: Vec<Vec<PlSmallStr>>,
+        aggs_per_input: Vec<Vec<PlSmallStr>>,
     },
     #[cfg(feature = "dynamic_group_by")]
     DynamicGroupBy {
@@ -197,6 +198,7 @@ pub enum PhysNodeProperties {
         row_index_offset: Option<u64>,
         pre_slice: Option<(i64, u64)>,
         predicate: Option<PlSmallStr>,
+        predicate_file_skip_applied: Option<PredicateFileSkip>,
         has_table_statistics: bool,
         include_file_paths: Option<PlSmallStr>,
         deletion_files_type: Option<PlSmallStr>,
@@ -265,10 +267,8 @@ pub enum PhysNodeProperties {
         num_sinks: u64,
     },
     Sort {
-        by_exprs: Vec<PlSmallStr>,
+        sort_columns: Vec<SortColumn>,
         slice: Option<(i64, u64)>,
-        descending: Vec<bool>,
-        nulls_last: Vec<bool>,
         multithreaded: bool,
         maintain_order: bool,
         limit: Option<u64>,
@@ -326,3 +326,14 @@ pub enum PhysNodeProperties {
         validate_schema: bool,
     },
 }
+
+#[derive(serde::Serialize, serde::Deserialize, Clone, Debug, PartialEq)]
+#[cfg_attr(
+    feature = "physical_plan_visualization_schema",
+    derive(schemars::JsonSchema)
+)]
+pub struct SortColumn {
+    pub expr: PlSmallStr,
+    pub descending: bool,
+    pub nulls_last: bool,
+}
