@@ -34,13 +34,13 @@ impl<T: Num> Default for KahanSum<T> {
 
 impl<T: IsFloat + Num + AddAssign + Copy> AddAssign<T> for KahanSum<T> {
     fn add_assign(&mut self, rhs: T) {
-        if rhs.is_finite() {
-            let y = rhs - self.err;
-            let new_sum = self.sum + y;
-            self.err = (new_sum - self.sum) - y;
-            self.sum = new_sum;
-        } else {
-            self.sum += rhs
+        let y = rhs - self.err;
+        let new_sum = self.sum + y;
+        let new_err = (new_sum - self.sum) - y;
+        self.sum = new_sum;
+        if new_err.is_finite() {
+            // Ensure err stays finite so we don't introduce NaNs through Inf - Inf.
+            self.err = new_err;
         }
     }
 }
