@@ -247,22 +247,17 @@ def test_join_inner_multi(join_clause: str, expected_error: str | None) -> None:
     }
     with pl.SQLContext(frames) as ctx:
         assert ctx.tables() == ["tbl_a", "tbl_b", "tbl_c"]
-        for select_cols in (
-            "a, b, c, d",
-            "tbl_a.a, tbl_a.b, tbl_b.c, tbl_c.d",
-        ):
-            query = f"""
-                SELECT {select_cols}
-                FROM tbl_a {join_clause}
-                ORDER BY a DESC
-            """
-            try:
-                out = ctx.execute(query)
-                assert out.collect().rows() == [(1, 4, "z", 25.5)]
+        query = f"""
+            SELECT tbl_a.a, tbl_a.b, tbl_b.c, tbl_c.d
+            FROM tbl_a {join_clause}
+            ORDER BY tbl_a.a DESC
+        """
+        try:
+            out = ctx.execute(query)
+            assert out.collect().rows() == [(1, 4, "z", 25.5)]
 
-            except SQLInterfaceError as err:
-                if expected_error and expected_error in str(err):
-                    break
+        except SQLInterfaceError as err:
+            if not (expected_error and expected_error in str(err)):
                 raise
 
 
@@ -286,7 +281,10 @@ def test_join_left_multi(join_clause: str) -> None:
         "tbl_c": pl.DataFrame({"c": ["w", "y", "z"], "d": [10.5, -50.0, 25.5]}),
     }
     with pl.SQLContext(frames) as ctx:
-        for select_cols in ("a, b, c, d", "tbl_a.a, tbl_a.b, tbl_b.c, tbl_c.d"):
+        for select_cols in (
+            "tbl_a.a, tbl_a.b, tbl_b.c, tbl_c.d",
+            "tbl_a.a, tbl_a.b, tbl_b.c, d",
+        ):
             out = ctx.execute(
                 f"SELECT {select_cols} FROM tbl_a {join_clause} ORDER BY a DESC"
             )
@@ -304,23 +302,23 @@ def test_join_left_multi_nested() -> None:
         "tbl_c": pl.DataFrame({"c": ["w", "y", "z"], "d": [10.5, -50.0, 25.5]}),
     }
     with pl.SQLContext(frames) as ctx:
-        for select_cols in ("a, b, c, d", "tbl_x.a, tbl_x.b, tbl_x.c, tbl_c.d"):
-            out = ctx.execute(
-                f"""
-                SELECT {select_cols} FROM (SELECT *
-                    FROM tbl_a
-                    LEFT JOIN tbl_b ON tbl_a.a = tbl_b.a AND tbl_a.b = tbl_b.b
-                ) tbl_x
-                LEFT JOIN tbl_c ON tbl_x.c = tbl_c.c
-                ORDER BY tbl_x.a ASC
-                """
-            ).collect()
+        out = ctx.execute(
+            """
+            SELECT tbl_x.a, tbl_x.b, tbl_x.c, tbl_c.d FROM (
+                SELECT *
+                FROM tbl_a
+                LEFT JOIN tbl_b ON tbl_a.a = tbl_b.a AND tbl_a.b = tbl_b.b
+            ) tbl_x
+            LEFT JOIN tbl_c ON tbl_x.c = tbl_c.c
+            ORDER BY tbl_x.a ASC
+            """
+        ).collect()
 
-            assert out.rows() == [
-                (1, 4, "z", 25.5),
-                (2, None, None, None),
-                (3, 6, "x", None),
-            ]
+        assert out.rows() == [
+            (1, 4, "z", 25.5),
+            (2, None, None, None),
+            (3, 6, "x", None),
+        ]
 
 
 def test_join_misc_13618() -> None:
@@ -769,7 +767,7 @@ def test_nested_joins_17381() -> None:
         SELECT *
         FROM a
         WHERE id IN (
-            SELECT id
+            SELECT a2.id
             FROM a
             INNER JOIN a AS a2 ON a.id = a2.id
         )
@@ -1211,3 +1209,23 @@ def test_unsupported_join_conditions(join_condition: str, expected_error: str) -
 
     with pytest.raises(SQLInterfaceError, match=expected_error):
         pl.sql(f"SELECT * FROM df1 INNER JOIN df2 ON {join_condition}")
+
+
+def test_ambiguous_column_detection_in_joins() -> None:
+    # unqualified column references that exist in multiple tables should raise
+    # an error (with a helpful suggestion about qualifying the reference)
+    with pytest.raises(
+        SQLInterfaceError,
+        match=r'ambiguous reference to column "k" \(use one of: a\.k, c\.k\)',
+    ):
+        pl.sql(
+            query="""
+                WITH a AS (
+                  SELECT 0 AS k
+                ), c AS (
+                  SELECT 0 AS k
+                )
+                SELECT k FROM a JOIN c ON a.k = c.k
+            """,
+            eager=True,
+        )
